"use strict";
/*!
 * Paradigm UI Web
 * Copyright (c) 2017 Miracle Devs, Inc
 * Licensed under MIT (https://gitlab.com/miracledevs-paradigm/ui-web-angularjs/blob/master/LICENSE)
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var directive_1 = require("../decorators/directive");
var base_directive_1 = require("./base.directive");
var key_processor_service_1 = require("../services/key-processor.service");
var $ = require("jquery");
var KeyboardListenerDirective = /** @class */ (function (_super) {
    tslib_1.__extends(KeyboardListenerDirective, _super);
    function KeyboardListenerDirective(keyProcessor) {
        var _this = _super.call(this) || this;
        _this.keyProcessor = keyProcessor;
        return _this;
    }
    KeyboardListenerDirective_1 = KeyboardListenerDirective;
    KeyboardListenerDirective.prototype.create = function (scope, instanceElement, instanceAttributes, controller, transclude) {
        var actions = "";
        var attachTo = $(scope.attachTo || document);
        instanceElement.find("listener").each(function (index, element) {
            actions += element.getAttribute("action") + "|";
        });
        if (actions[actions.length - 1] === "|")
            actions = actions.substr(0, actions.length - 1);
        var keyActions = this.keyProcessor.parseActions(actions);
        var keyProcessor = this.keyProcessor;
        function evaluateKeyPress(e) {
            if (scope.disabled)
                return;
            keyProcessor.evaluateKeyActions(keyActions, "keypress", scope.$parent, e);
        }
        function evaluateKeyDown(e) {
            if (scope.disabled)
                return;
            keyProcessor.evaluateKeyActions(keyActions, "keydown", scope.$parent, e);
        }
        function evaluateKeyUp(e) {
            if (scope.disabled)
                return;
            keyProcessor.evaluateKeyActions(keyActions, "keyup", scope.$parent, e);
        }
        if (keyActions.any(function (x) { return x.eventType === "keypress"; }))
            attachTo.on("keypress.documentKeyboard", evaluateKeyPress);
        if (keyActions.any(function (x) { return x.eventType === "keydown"; }))
            attachTo.on("keydown.documentKeyboard", evaluateKeyDown);
        if (keyActions.any(function (x) { return x.eventType === "keyup"; }))
            attachTo.on("keyup.documentKeyboard", evaluateKeyUp);
    };
    KeyboardListenerDirective.prototype.dispose = function (scope, instanceElement, instanceAttributes, controller, transclude) {
        var attachTo = $(scope.attachTo || document);
        attachTo.off("keypress.documentKeyboard");
        attachTo.off("keydown.documentKeyboard");
        attachTo.off("keyup.documentKeyboard");
        _super.prototype.dispose.call(this, scope, instanceElement, instanceAttributes, controller, transclude);
    };
    KeyboardListenerDirective.factory = function (keyProcessor) {
        return new KeyboardListenerDirective_1(keyProcessor);
    };
    KeyboardListenerDirective = KeyboardListenerDirective_1 = tslib_1.__decorate([
        directive_1.Directive({
            name: "keyboardListener",
            restrict: "E",
            bindings: {
                disabled: "=",
                attachTo: "@"
            },
            dependencies: [key_processor_service_1.KeyProcessorService]
        })
    ], KeyboardListenerDirective);
    return KeyboardListenerDirective;
    var KeyboardListenerDirective_1;
}(base_directive_1.DirectiveBase));
exports.KeyboardListenerDirective = KeyboardListenerDirective;
