"use strict";
/*!
 * Paradigm UI Web
 * Copyright (c) 2017 Miracle Devs, Inc
 * Licensed under MIT (https://gitlab.com/miracledevs-paradigm/ui-web-angularjs/blob/master/LICENSE)
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var service_1 = require("../decorators/service");
var angular_service_1 = require("./angular.service");
var base_service_1 = require("./base.service");
var paradigm_ui_web_shared_1 = require("@miracledevs/paradigm-ui-web-shared");
var angular = require("angular");
var registrable_1 = require("../decorators/registrable");
var $ = require("jquery");
var ModalService = /** @class */ (function (_super) {
    tslib_1.__extends(ModalService, _super);
    function ModalService($rootScope, $q, $http, $templateCache, $compile, $controller) {
        var _this = _super.call(this) || this;
        _this.$rootScope = $rootScope;
        _this.$q = $q;
        _this.$http = $http;
        _this.$templateCache = $templateCache;
        _this.$compile = $compile;
        _this.$controller = $controller;
        _this.modals = new paradigm_ui_web_shared_1.Dictionary();
        return _this;
    }
    ModalService_1 = ModalService;
    ModalService.prototype.open = function (dialog, parameters, staticDialog, keyboard) {
        var _this = this;
        if (staticDialog === void 0) { staticDialog = false; }
        if (keyboard === void 0) { keyboard = true; }
        var dialogInfo = paradigm_ui_web_shared_1.ObjectExtensions.getTypeName(dialog) === "Function" ? dialog[registrable_1.registrableMetadataKey] : dialog;
        if (dialogInfo == null)
            throw new Error("The modal controller registration information is missing.");
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(dialogInfo.template) &&
            paradigm_ui_web_shared_1.ObjectExtensions.isNull(dialogInfo.templateUrl))
            throw new Error("Can not obtain a template for the modal. Use either template or templateUrl");
        var controllerAs = dialogInfo.controllerAs || "controller";
        var modalInstance = new ModalInstance(this, this.$q.defer());
        var template = !paradigm_ui_web_shared_1.ObjectExtensions.isNull(dialogInfo.template) ? dialogInfo.template : this.$templateCache.get(dialogInfo.templateUrl);
        // TODO: search for a better way to handle templateCache. $http probably have a handler to store the result on cache instead of storning the whole result object.
        if (template == null)
            this.$http.get(dialogInfo.templateUrl, { cache: this.$templateCache }).then(function (result) { return _this.openModal(dialogInfo, parameters, modalInstance, result.data, staticDialog, keyboard); });
        else
            this.openModal(dialogInfo, parameters, modalInstance, paradigm_ui_web_shared_1.ObjectExtensions.getTypeName(template).toLowerCase() === "string" ? template : template[1], staticDialog, keyboard);
        return modalInstance;
    };
    ModalService.prototype.close = function (modalInstance, reason) {
        var modal = this.modals.get(modalInstance);
        if (modal == null)
            return;
        var modalElement = $(modal);
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(modalElement["modal"]))
            return;
        modalElement["modal"]("hide");
        modalInstance.deferred.reject(reason);
    };
    ModalService.prototype.resolve = function (modalInstance, result) {
        var modal = this.modals.get(modalInstance);
        if (modal == null)
            return;
        var modalElement = $(modal);
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(modalElement["modal"]))
            return;
        modalElement["modal"]("hide");
        modalInstance.deferred.resolve(result);
    };
    ModalService.prototype.openModal = function (dialogInfo, parameters, modalInstance, template, staticDialog, keyboard) {
        var _this = this;
        // create a new scope for the modal dialog.
        var scope = this.$rootScope.$new(true);
        var controllerParameters = {};
        controllerParameters[angular_service_1.AngularServices.scope] = scope;
        controllerParameters[paradigm_ui_web_shared_1.FunctionExtensions.getFunctionName(ModalInstance)] = modalInstance;
        controllerParameters[exports.ModalParameters] = parameters;
        // instantiate the modal controller.
        var controller = this.$controller(dialogInfo.controller, controllerParameters);
        // set the controller alias (by default will be controller).
        scope[dialogInfo.controllerAs] = controller;
        // create the modal DOM elements.
        var id = paradigm_ui_web_shared_1.Guid.new().value;
        var modalBody = angular.element("<div class=\"modal fade\" id=\"" + id + "\" tabindex=\"-1\" role=\"dialog\" aria-labelledby=\"login-title\"></div>");
        modalBody.html(template);
        // compile the modal DOM for angular to resolve bindings and elements.
        var code = this.$compile(modalBody)(scope);
        // add the modal DOM to the page body.
        $("body").append(code);
        // register the modal inside the service.
        this.modals.add(modalInstance, code);
        // setup the destroy event to clean the DOM.
        scope.$on("$destroy", function () {
            code.remove();
        });
        var dialogOptions = {};
        if (staticDialog)
            dialogOptions["backdrop"] = "static";
        if (keyboard)
            dialogOptions["keyboard"] = keyboard;
        // open the MaterializeCSS modal.
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(code["modal"]))
            throw new Error("The modal method is not registered and the modal window can not be opened.");
        code["modal"](dialogOptions);
        // set the z-index of modal and backdrop
        var zIndex = this.modals.getValues().count() * 1050;
        code.css("z-index", zIndex);
        $(code[0].nextElementSibling).css("z-index", zIndex - 10);
        $(code).on("hidden.bs.modal", function () {
            modalInstance.deferred.reject("cancelled");
            _this.removeModal(modalInstance, $(code));
        });
    };
    ModalService.prototype.removeModal = function (modalInstance, modal) {
        var scope = modal.scope();
        if (modalInstance.dispose != null)
            modalInstance.dispose();
        this.modals.remove(modalInstance);
        if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(scope) && !paradigm_ui_web_shared_1.ObjectExtensions.isNull(scope["controllerAs"])) {
            delete scope["controllerAs"];
            scope.$destroy();
        }
        if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(modal)) {
            modal.remove();
        }
    };
    ModalService.factory = function ($rootScope, $q, $http, $templateCache, $compile, $controller) {
        return new ModalService_1($rootScope, $q, $http, $templateCache, $compile, $controller);
    };
    ModalService = ModalService_1 = tslib_1.__decorate([
        service_1.Service({
            name: "$pd-modalService",
            dependencies: [
                angular_service_1.AngularServices.rootScope,
                angular_service_1.AngularServices.q,
                angular_service_1.AngularServices.http,
                angular_service_1.AngularServices.templateCache,
                angular_service_1.AngularServices.compile,
                angular_service_1.AngularServices.controller
            ]
        })
    ], ModalService);
    return ModalService;
    var ModalService_1;
}(base_service_1.ServiceBase));
exports.ModalService = ModalService;
var ModalInstance = /** @class */ (function () {
    function ModalInstance(modalService, deferred) {
        this.modalService = modalService;
        this.deferred = deferred;
        this.promise = deferred.promise;
    }
    ModalInstance.prototype.close = function (reason) {
        this.modalService.close(this, reason);
    };
    ModalInstance.prototype.resolve = function (result) {
        this.modalService.resolve(this, result);
    };
    return ModalInstance;
}());
exports.ModalInstance = ModalInstance;
exports.ModalParameters = "ModalParameters";
